! CVS: $Id$
! CVS: $Source$
! CVS: $Name$
!BOP
!
! !MODULE: ice_mechred - computes mech redistribution and strength for ITD
!
! !DESCRIPTION:
!
! Ice thickness distribution model with multi-layer thermodynamics.
! Routines to compute mechanical redistribution and strength. \\
!
! See Bitz, C.M., and W.H. Lipscomb, 1999: 
! An energy-conserving thermodynamic model of sea ice,
! J. Geophys. Res., 104, 15,669-15,677. \\
!     
! See Bitz, C.M., M.M. Holland, A.J. Weaver, M. Eby, 2001: 
! Simulating the ice-thickness distribution in a climate model,
! J. Geophys. Res., 106, 2441--2464. \\
!
! !REVISION HISTORY:
!
! author: C. M. Bitz
!
! !INTERFACE:
!
      module ice_mechred
!
! !USES:
!
      use ice_model_size
      use ice_constants
      use ice_state
      use ice_itd
      use ice_dyn_evp
!
!EOP
!
      implicit none

      logical (kind=log_kind) ::
     &   snow_into_ocn  ! if true, throw snow into ocean
     &,  asum_error        ! flag for asum .ne. 1

      real (kind=dbl_kind), dimension (ilo:ihi,jlo:jhi) ::
     &   closing  ! lead closing rate         (1/s)
     &,  divu_tr  ! actual divergence rate, computed from transport (1/s)

      ! default redistributers, when thickness is at category mid-point
      real (kind=dbl_kind), dimension (ncat,ncat) ::
     &   Gammdf   !     area frac from cat i that goes into j
     &,  HGammdf  !   volume frac from cat i that goes into j
     &,  H2Gammdf ! H*volume frac from cat i that goes into j

      integer (kind=int_kind) ::
     &   ncrel    (ncat,ncat) ! helps transfer heat when ridging
     &,  kstrength            ! 1 for Rothrock 1975 strength formulation
     &,  kcolumn              ! 1 for column model

      real (kind=dbl_kind), parameter ::
     &   gstar = 0.15_dbl_kind ! if open water area > GSTAR then no 
                               ! mechanical redist.
     &,  Zfric = 17.0_dbl_kind ! frictional energy mult. factor for 
                               ! Roth. 75 strength 
     &,  Pstar = 2.75e4_dbl_kind ! empirical formula if not using 
                                 ! Roth. 75 strength  (N/m**2)
     &,  cK = 100.0_dbl_kind  ! max ridged ice thickness is 2*sqrt(cK*Hi) 
                              ! where Hi is orig thickness (m)
     &,  cpe = 450._dbl_kind ! if using Rothrock75 energetics argument for 
                    ! strength, coef. for potential energy term    (N/m**3)

!
      contains
!
!BOP
!
! !IROUTINE: init_mechred - initialize constants for ridging
!
! !INTERFACE: 
!
      subroutine init_mechred
!
! !DESCRIPTION:
!
! Initialize constants for ridging
!
! !REVISION HISTORY:
!
! author: C.M.Bitz, UW
!
! !USES:
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      integer (kind=int_kind) :: nc,k,l,m,layer, i,j
      logical (kind=log_kind) :: rowflg(ncat) ! flag for ridg. matrix row
      real (kind=dbl_kind) ::
     &   hin       (ncat)     ! ice thickness                        (m)
     &,  Hmean    (ncat)     ! a dummy variable at setup            (m)

      !-----------------------------------------------------------------
      ! default ridg matrices,  comp. all rows (rowflg=true)
      !-----------------------------------------------------------------
      ! assume ice thickness is mean of range for nc < ncat
      ! and 1m thicker than lower limit for ncat
      !-----------------------------------------------------------------
      do nc = 1,ncat
         do k = 1,ncat
            Gammdf(nc,k)   = c0
            HGammdf(nc,k)  = c0
            H2Gammdf(nc,k) = c0
         enddo
         rowflg(nc) = .true.  
         if (nc.lt.ncat) then
            hin(nc) = p5*(hin_max(nc-1) + hin_max(nc))
         else
            hin(ncat) = hin_max(ncat-1) + c1
         endif
      enddo
         
      call comp_matrices (    rowflg ,hin      ,Hmean
     $,                Gammdf ,HGammdf ,H2Gammdf )

      !-----------------------------------------------------------------
      ! initialize deformation and strength arrays
      !-----------------------------------------------------------------
      do j=jlo,jhi
      do i=ilo,ihi
        strength(i,j)=c0
        closing(i,j)=c0
        divu_tr(i,j)=c0
      enddo
      enddo

      end subroutine init_mechred

!=======================================================================
!BOP
!
! !IROUTINE: ridge - ridge ice due to convergence or shear deformation
!
! !INTERFACE:
!
      subroutine ridge
!
! !DESCRIPTION:
!
! Ridging  works with ncat=1 \\
! In fact, it will provide a source of open water from shear 
! deformation. See Stern et al, 1995 for information about 
! how and why this is done \\
!
! Keeps track of heat and fresh water flx to ocn \\
! if snow on ice that ridges is thrown into the ocean \\

! Ridge ice due to convergence or shear deformation \\
! note the timestep is dyn\_dt for ridging among the cats  \\
! however will subcycle with timestep dtsub if a cat   \\
! would run out of ice if the timestep were greater than dtsub   \\
!
! Throw snow on ice that ridges into ocean if snow\_into\_ocn=.true. \\
!
! !REVISION HISTORY:
!
! author: C.M.Bitz, UW
!
! !USES:
!
      use ice_domain 
      use ice_flux
      use ice_grid
      use ice_timers
      use ice_exit
      use ice_calendar, only: dt, dyn_dt, istep1
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      integer (kind=int_kind) :: i, j, k, nc, layer, l

      integer (kind=int_kind), dimension (1:(ihi-ilo+1)*(jhi-jlo+1)) ::
     &  indxi                 ! compressed indices in i/j directions
     &, indxj

      integer (kind=int_kind) ::
     &  icells                ! number of ocean/ice cells
     &, ij                    ! combined i/j horizontal index

      real (kind=dbl_kind) :: aice_jls   ! JLS

      real (kind=dbl_kind), dimension (ilo:ihi,jlo:jhi) ::
     &   asum                ! total fraction of ice + open water

      real (kind=dbl_kind) ::
     &   dtsub               ! subcycling timestep                     (s)
     &,  dtsubn              ! used to find subcycling timestep        (s)
     &,  dtaccum             ! accumulated time in this routine        (s)
     &,  dvisum              ! sum of dvi                              (m)
     &,  dvssum              ! sum of dvs                              (m)
     &,  deisum              ! sum of dei                              (J)
     &,  clsdt               ! closing(i,j) * timestep
     &,  divudt              ! divergence * timestep
     &,  Gamm    (ncat,ncat) ! area frac from cat i that goes into j
     &,  H2Gamm  (ncat,ncat) ! H*volume frac from cat i that goes into j

      real (kind=dbl_kind) ::
         !  if using empirical formula for computing strength
     &   ailcl               ! local sum of ice fraction over categories
     &,  vilcl               ! local sum of ice volume over categories (m)

      real (kind=dbl_kind), dimension (0:ncat) ::
     &   W                   ! W factor, geometric comp. of ridging mode 
     &,  Wa                  ! part of W from participation func.
     &,  Psi                 ! rate of chng of area frac from ridg   (1/s)

      real (kind=dbl_kind), dimension (ncat,ncat) ::
     &   HGamm               ! volume frac from cat i that goes into j

      real (kind=dbl_kind), dimension (ncat) ::
     &   Hmean               ! mean thickness of ridged ice from cat nc
     &,  hin                  ! ice thickness                           (m)
     &,  hsn                  ! snow thickness                          (m)
     &,  dvi                 ! geometric factor for ice volume         (m)
     &,  dvs                 ! geometric factor for snow volume        (m)
     &,  dtf                 ! geometric factor, temp * frac area      (K)
     &,  ai_Tsfn              ! temperature * fractional area           (K)

      real (kind=dbl_kind), dimension (nilyr,ncat) ::
     &   qin                  ! energy of melt ice per unit vol.   (J/m**3)
     &,  dei                 ! geometric factor for energy of melt     (J)

      call ice_timer_start(6)  ! ridging 

      asum_error = .false.

      icells = 0
      do j = jlo, jhi
      do i = ilo, ihi
        if (tmask(i,j)) then
          icells = icells + 1
          indxi(icells) = i
          indxj(icells) = j
        endif   ! tmask
      enddo
      enddo

      do ij = 1, icells
         i = indxi(ij)
         j = indxj(ij)

         dtsub   = dyn_dt  ! begin assuming the subcyling step is the full timestep
         dtaccum = 0 

         if (kcolumn.ne.1) then 
         !----------------------------------------------------------------
         ! compute opening alpha_o and closing rate alpha_r
         ! from divergence/shear quantities 
         ! see Thorndike et al., 1975 and Flato and Hibler, 1995
         !----------------------------------------------------------------
         ! It is absolutely essential that the divergence used in this
         ! routine be equal to the actual rate that ice has diverged into
         ! a grid cell. Ideally we would like this to be equal to the
         ! divergence of the ice velocity field (divu).
         ! However, some transport schemes, like MPDATA, have higher
         ! order correction to the transport which require adjusting
         ! the computation of divu in non-obvious ways.
         ! Instead let the divergence used here be computed
         ! directly from the ice concentration and
         ! ignore the effects on Delta, which is less critical.
         !-----------------------------------------------------------------
           asum(i,j) = aice0(i,j)
           do nc = 1,ncat
             asum(i,j) = asum(i,j) + aicen(i,j,nc)
           enddo

           divu_tr(i,j) = ( c1 - asum(i,j) )/dyn_dt

         ! following Stern et al., 1995     - MAX SHEAR DEFORMATION
!           closing(i,j) = p5*(Delta(i,j) - divu_tr(i,j)) 
!           closing(i,j) = max(closing(i,j), -divu_tr(i,j))

         ! following Flato Hibler 95        - INTERM. SHEAR DEFORMATION
!           closing(i,j) = p25*(Delta(i,j)-abs(divu_tr(i,j))) - min(divu_tr(i,j),c0)
!           closing(i,j) = max(closing(i,j), -divu_tr(i,j))

         ! Bill Hibler suggesting           - NO SHEAR DEFORMATION
!           closing(i,j) = -min(divu_tr(i,j), c0)

         ! BETWEEN INTERM. and NO SHEAR DEFORMATION
           closing(i,j) = 0.1125_dbl_kind*(Delta(i,j) - abs(divu_tr(i,j)))
     &          -min(divu_tr(i,j), c0)
           closing(i,j) = max(closing(i,j), -divu_tr(i,j))

         else

         !-----------------------------------------------------------------
         ! divu_tr(i,j) and closing(i,j) must be prescribed (i.e. read in 
         ! from a file) for Column model.  It is currently set to 0 above.
         ! Also must add the g(h) div u term to ITD
         !-----------------------------------------------------------------

           divudt = divu_tr(i,j)*dyn_dt

           if (divudt.gt.1.0) then
             print*, 'ice_mechred divudt>1!'
             call abort_ice ('(ridge) divudt > 1')
           endif

           aice0(i,j) = aice0(i,j) - divudt*aice0(i,j)

           do nc = 1,ncat
             aicen(i,j,nc) = aicen(i,j,nc) - divudt*aicen(i,j,nc)
             vicen(i,j,nc) = vicen(i,j,nc) - divudt*vicen(i,j,nc)
             vsnon(i,j,nc) = vsnon(i,j,nc) - divudt*vsnon(i,j,nc)
             do layer = 1,nilyr
               eicen(i,j,ilyr1(nc)+layer-1) = eicen(i,j,ilyr1(nc)+layer-1)
     &                               - divudt*eicen(i,j,ilyr1(nc)+layer-1)
             enddo
           enddo

           asum(i,j) = aice0(i,j)
           do nc = 1,ncat
             asum(i,j) = asum(i,j) + aicen(i,j,nc)
           enddo
         endif         ! kcolumn

         !-----------------------------------------------------------------
         ! increase open water and asum by opening rate
         !-----------------------------------------------------------------
         aice0(i,j) = aice0(i,j)  + dyn_dt*(divu_tr(i,j) + closing(i,j))
         asum(i,j)  = asum(i,j)   + dyn_dt*(divu_tr(i,j) + closing(i,j))

         if (asum(i,j) <= c1) then  ! increase open water and go to next point

           aice0(i,j) = aice0(i,j) + c1 - asum(i,j) 

           asum(i,j) = aice0(i,j)   ! Update asum for error checking
           do nc = 1,ncat
             asum(i,j) = asum(i,j) + aicen(i,j,nc)
           enddo
           if (kstrength.eq.1 .and. aice0(i,j).le.Gstar) 
     &        call ridge_matrices( aicen(i,j,:), vicen(i,j,:),     ! JLS
     &                            hin, Hmean, Gamm, HGamm, H2Gamm ) 
           goto 100
         endif

      ! if asum > 1 must ridge

 50   continue      

      !-----------------------------------------------------------------
      ! compute the ridge matrices and W factors, see docum.
      !-----------------------------------------------------------------
         call ridge_matrices( aicen(i,j,:), vicen(i,j,:), hin,  ! JLS
     &                        Hmean, Gamm, HGamm, H2Gamm ) 
         call ridging_mode ( aice0(i,j), aicen(i,j,:), W, Wa, Gamm )  ! JLS

c      print*,'Wa ', (Wa(nc),nc=0,ncat)
c      print*,'W  ', (W(nc),nc=0,ncat)

         if (ncat.gt.1) then
         do nc = 1,ncat
            hsn(nc) = c0
            do layer = 1,nilyr
               qin(layer,nc) = c0
            enddo
            if ((aicen(i,j,nc).gt.puny) .and. (vicen(i,j,nc).gt.0.))  then
               hsn(nc) = vsnon(i,j,nc)/aicen(i,j,nc)
               do layer = 1,nilyr
                  qin(layer,nc) = eicen(i,j,ilyr1(nc)+layer-1)*nilyr/vicen(i,j,nc)
               enddo
            endif
         enddo

      !-----------------------------------------------------------------
      ! compute terms in bracket of Theta (geom. term, see docum.)
      !-----------------------------------------------------------------
         dvisum = c0
         dvssum = c0
         do nc = 1,ncat-1
            dvi(nc) = -hin(nc)*Wa(nc)
            dvs(nc) = -hsn(nc)*Wa(nc)
            do k = 1,nc
               dvi(nc) = dvi(nc) + Wa(k)*HGamm(k,nc)
               if (.not. snow_into_ocn)
     &         dvs(nc) = dvs(nc) + Wa(k)*HGamm(k,nc)*hsn(k)/hin(k)
            enddo
            dvisum = dvisum + dvi(nc)
            dvssum = dvssum + dvs(nc)
         enddo
 
         dvi(ncat) = -dvisum       ! guarantees ice volume conservation
         if (snow_into_ocn) then
           dvs(ncat) = -hsn(ncat)*Wa(ncat) ! only participation term 
           dvssum = dvssum + dvs(ncat)
         else
           dvs(ncat) = -dvssum       ! guarantees snow volume conservation
         endif

      !-----------------------------------------------------------------
      ! compute terms in bracket of Pi (geom. term, see docum.)
      !-----------------------------------------------------------------
      ! when ice ridges from cat k into cat nc heat is 
      ! transf. horiz. by layer w/o vert. mixing
      ! e.g., if cat k has 2 layers and cat nc has 4, heat from
      ! layer 1 of cat k is split evenly btwn layers 1 & 2 of cat nc
      !-----------------------------------------------------------------
         deisum = c0
         do nc = 1,ncat-1
            do layer = 1,nilyr
               dei(layer,nc) = -Wa(nc)*hin(nc)*qin(layer,nc)/nilyr
               do k = 1,nc
                  dei(layer,nc) = dei(layer,nc)+
     $                 qin(layer,k)*Wa(k)*HGamm(k,nc)/nilyr
               enddo
               deisum = deisum+dei(layer,nc)
            enddo
         enddo
         do layer = 1,nilyr
            dei(layer,ncat) = c0
            do k = 1,ncat-1
               dei(layer,ncat) = dei(layer,ncat)+
     $              qin(layer,k)*Wa(k)*HGamm(k,ncat)/nilyr
            enddo
            deisum = deisum+dei(layer,ncat)
         enddo
c        write(*,*) 'Energy is conserved if deisum is zero: deisum= '
c    $       ,deisum

      !-----------------------------------------------------------------
      ! geometrical factor for treating ice/snow surface temp when ridging
      !-----------------------------------------------------------------
         do nc = 1,ncat
            dtf(nc) = - Tsfcn(i,j,nc)*Wa(nc)
            do k = 1,nc
              dtf(nc) = dtf(nc) + Tsfcn(i,j,k)*Wa(k)*Gamm(k,nc)
            enddo
         enddo

         endif ! ncat > 1

      !-----------------------------------------------------------------
      ! make sure that only ridge as much ice as is available, 
      ! otherwise reduce timestep
      !-----------------------------------------------------------------
         Psi(0) = closing(i,j)*Wa(0)
         if ((Psi(0)*dtsub.gt.aice0(i,j)) .and. (Psi(0).gt.0.)) then
            dtsubn = aice0(i,j)/Psi(0)
            dtsub  = min(dtsub,dtsubn)
         endif
         do nc = 1,ncat
            Psi(nc) = closing(i,j)*Wa(nc)
            if ((Psi(nc)*dtsub.gt.aicen(i,j,nc)) .and. (Psi(nc).gt.0.)) then
               dtsubn = aicen(i,j,nc)/Psi(nc)
               dtsub  = min(dtsub,dtsubn)
            endif
         enddo

      !-----------------------------------------------------------------
      ! finally update the prognostic variables
      !-----------------------------------------------------------------
         clsdt = closing(i,j)*dtsub 

c         print*, 'clsdt',clsdt,closing(i,j),dtsub,dyn_dt
         aice0(i,j)   = aice0(i,j) + clsdt*W(0)
         asum(i,j)  = aice0(i,j)
         do nc = 1,ncat
            if (ncat.gt.1) then
              ai_Tsfn(nc) = aicen(i,j,nc)*Tsfcn(i,j,nc) + clsdt*dtf(nc)
            endif
            aicen(i,j,nc)  = aicen(i,j,nc) + clsdt*W(nc)

            asum(i,j) = asum(i,j) + aicen(i,j,nc)
            if (ncat.gt.1) then
              vicen(i,j,nc)  = vicen(i,j,nc) + clsdt*dvi(nc)
              vsnon(i,j,nc)  = vsnon(i,j,nc) + clsdt*dvs(nc)
              esnon(i,j,nc)  = -vsnon(i,j,nc)*rLfs	! MH
              if (snow_into_ocn) then
         !--------------------------------------------------------------
         ! Assuming snow falls into the ocn
         ! dvs is the volume of snow on ice that participates in ridging
         ! dvs <= 0.  
         ! add a positive quantity to Fresh(i,j) (ocn gains water from snow)
         ! add a negative quantity to Fhnet(i,j) (ocn looses heat to melt snow)
         !--------------------------------------------------------------
                Fresh(i,j)      = Fresh(i,j)   - clsdt*dvs(nc)*rhos/dt !  kg/m^2 s
                Fresh_hist(i,j) = Fresh_hist(i,j) - clsdt*dvs(nc)*rhos/dt !  kg/m^2 s
                Fhnet(i,j)      = Fhnet(i,j)   + clsdt*dvs(nc)*rLfs/dt   
                Fhnet_hist(i,j) = Fhnet_hist(i,j) + clsdt*dvs(nc)*rLfs/dt   
              endif
              do layer = 1,nilyr
                eicen(i,j,ilyr1(nc)+layer-1) = eicen(i,j,ilyr1(nc)+layer-1) + 
     &                                         clsdt*dei(layer,nc)
              enddo
            endif ! ncat > 1
         enddo

         if (ncat.gt.1) then
           do nc = 1,ncat
             if (aicen(i,j,nc).gt.0.) then
               Tsfcn(i,j,nc) = ai_Tsfn(nc)/aicen(i,j,nc)
             else
               Tsfcn(i,j,nc) = Tf(i,j)
             endif
           enddo
         endif ! ncat > 1

      !-----------------------------------------------------------------
      ! decide if time accumulated in ridge is sufficient to return
      !-----------------------------------------------------------------
         dtaccum = dtaccum + dtsub
         dtsub   = dyn_dt - dtaccum

         if (dtsub.gt.puny) go to 50

 100     continue

         if (kstrength.eq.1) then
           strength(i,j) = roth_strength(aice0(i,j), aicen(i,j,:),vicen(i,j,:),
     &                     Gamm, H2Gamm )   ! JLS
         else
           ailcl = c0
           vilcl = c0
           do nc = 1,ncat
             ailcl = ailcl+aicen(i,j,nc)
             vilcl = vilcl+vicen(i,j,nc)
           enddo
           ! Hibler79, Eq 17, P=Pstar*h*exp(-C*(1-A)), h=effective thick 
           strength(i,j) = Pstar*vilcl*exp(-c20*(1-ailcl)) 
         endif

         if (abs(asum(i,j) - c1) > puny) asum_error = .true.

      enddo                    !ij

      if (asum_error) then
        do ij = 1, icells
          i = indxi(ij)
          j = indxj(ij)
          if (abs(asum(i,j) - c1) > puny) then ! there is a bug
            write(nu_diag,*) ''
            write(nu_diag,*) 'Ridging error: total area =', asum(i,j)
            write(nu_diag,*) 'istep1, my_task, i, j:',
     &                  istep1, my_task, i, j
            write(nu_diag,*) 'n, aice:',aicen(i,j,:)
            write(nu_diag,*)  0, aice0(i,j)
            call abort_ice ('ice: Ridging error, asum > 1')
          endif
        enddo
      endif

      call ice_timer_stop(6)   ! ridging 

      end subroutine ridge
!
!BOP
!
! !IROUTINE: ridging_mode - compute W and Wa factors 
!
! !INTERFACE:
!
      subroutine ridging_mode( ai0, ain, W, Wa, Gamm )
!
! !DESCRIPTION:
!
! Compute W and Wa, factors for ridging mode 
! and participation function
!
! !REVISION HISTORY:
!
! author: C.M.Bitz, UW
!
! !USES:
!
! !INPUT/OUTPUT PARAMETERS:
!
      real (kind=dbl_kind), intent(in) ::
     &  Gamm    (ncat,ncat) ! area frac from cat i that goes into j
     &, ai0                 ! JLS
     &, ain    (ncat)       ! JLS

      real (kind=dbl_kind), intent(out) ::
     &   W       (0:ncat)   ! W factor, geometric comp. of ridging mode 
     &,  Wa      (0:ncat)   ! part of W from participation func.
!
!EOP
!
      integer (kind=int_kind) ::  nc, k
      real (kind=dbl_kind) ::
     &   asum                ! sum of frac of ice + open water
     &,  Wsum                ! sum of W factors, for norm.
     &,  etar                ! norm. coef.
     &,  AUX(-1:ncat+1)      ! auxilary function

      !-----------------------------------------------------------------
      ! the Wa factor for cat nc is the area under the b(G) curve
      ! between G(nc-1) and G(nc) where G(nc) is the sum of ice 
      ! fractions upto and including cat nc, with G(-1)=0
      !-----------------------------------------------------------------

      AUX(-1)=c1

      AUX(0)=c0
      asum=ai0
      if (asum.lt.GSTAR)   AUX(0)=(1-asum/GSTAR)**2
      do nc=1,ncat
         AUX(nc)=c0
         asum=asum+ain(nc)
         if (asum.lt.GSTAR) AUX(nc)=(1-asum/GSTAR)**2
      enddo

      do nc=0,ncat
         Wa(nc)=AUX(nc-1)-AUX(nc)
      enddo

      !-----------------------------------------------------------------
      ! normalize W and Wa
      !-----------------------------------------------------------------
      Wsum = -Wa(0)
      do nc = 1,ncat
         W(nc) = -Wa(nc)
         do k = 1,nc
            W(nc) = W(nc) + Wa(k)*Gamm(k,nc)
         enddo
         Wsum = Wsum + W(nc)
      enddo

      etar = -c1/Wsum  ! normal. coef keep sign of Wa positive
      
      do nc=1,ncat
         W(nc)  = etar*W(nc)
         Wa(nc) = etar*Wa(nc)
      enddo
      Wa(0) = etar*Wa(0)

      W(0)  = -Wa(0)

      end subroutine ridging_mode
!
!BOP
!
! !IROUTINE: ridge_matrices - compute Gamma factors
!
! !INTERFACE:
!
      subroutine ridge_matrices(ain, vin, hin, Hmean, Gamm, HGamm, H2Gamm ) 
!
! !DESCRIPTION:
!
! Compute Gamm,HGamm,H2Gamm for ridge and strength for true thicknesses
!
! !REVISION HISTORY:
!
! author: C.M.Bitz, UW
!
! !USES:
!
! !INPUT/OUTPUT PARAMETERS:
!
      real (kind=dbl_kind), intent(in) ::
     &   ain      (ncat)      ! ice area    ! JLS
     &,  vin      (ncat)      ! ice volume  ! JLS
      real (kind=dbl_kind), intent(out) ::
     &   hin      (ncat)      ! ice thickness                           (m)
     &,  Hmean   (ncat)       ! mean thickness of ridg ice from cat nc  (m)
     &,  Gamm    (ncat,ncat)  ! area frac from cat i that goes into j
     &,  HGamm   (ncat,ncat)  ! volume frac from cat i that goes into j
     &,  H2Gamm  (ncat,ncat)  ! H*volume frac from cat i that goes into j
       ! HGamm is used in ridge and H2Gamm is used in roth_strength
!
!EOP
!
      integer (kind=int_kind) :: k,nc
      logical (kind=log_kind) :: rowflg(ncat) ! true if cat has nonzero area

      do nc = 1,ncat
         if ((ain(nc).gt.puny) .and. (vin(nc).gt.0.))  then
            hin(nc) = vin(nc)/ain(nc)
            rowflg(nc) = .true.
            do k = 1,ncat
              Gamm(nc,k)   = c0
              HGamm(nc,k)  = c0
              H2Gamm(nc,k) = c0
            enddo
         else
            hin(nc) = max(hin_max(nc-1),hi_min)
            rowflg(nc) = .false.
            do k = 1,ncat
              Gamm(nc,k)   = Gammdf(nc,k)
              HGamm(nc,k)  = HGammdf(nc,k)
              H2Gamm(nc,k) = H2Gammdf(nc,k)
            enddo
         endif
      enddo
         
      call comp_matrices  ( rowflg, hin, Hmean,
     $                      Gamm, HGamm, H2Gamm )

      end subroutine ridge_matrices
!
!BOP
!
! !IROUTINE: comp_matrices - compute Gamm functions
!
! !INTERFACE:
!
      subroutine comp_matrices( rowflg, hin, Hmean,
     $                           Gamm, HGamm, H2Gamm )
!
! !DESCRIPTION:
!
! Compute Gamm, HGamm, H2Gamm \\
! HGamm is used in ridge and H2Gamm is used in roth-strength  \\
! Gamm(nc,k) distributes ice that participates in ridging 
! from cat nc into a distrib. of cats k \\
!
! n1 is the smallest k with nonzero fract  \\
! n2 is the largest  k with nonzero fract  \\
!
! ice that partic. has thickness hin(nc) 
! and it ridges up to linear distrib. between
! 2*hin(nc) and 2*sqrt(cK*hin(nc))
!
! !REVISION HISTORY:
!
! author: C.M.Bitz, UW
!
! !USES:
!
! !INPUT/OUTPUT PARAMETERS:
!
      logical (kind=log_kind), intent(in) ::
     &   rowflg(ncat)     ! true if cat has nonzero fractional area
      real (kind=dbl_kind), intent(in) ::
     &   hin      (ncat)      ! ice thickness                           (m)

      real (kind=dbl_kind), intent(out) ::
     &   Hmean   (ncat)      ! mean thickness of ridg ice from cat nc  (m)
     &,  Gamm    (ncat,ncat) ! area frac from cat i that goes into j
     &,  HGamm   (ncat,ncat) ! volume frac from cat i that goes into j
     &,  H2Gamm  (ncat,ncat) ! H*volume frac from cat i that goes into j
!
!EOP
!
      integer (kind=int_kind) :: 
     &   nc, k
     &,  n1, n2           ! lowest and highest cat to which ice ridges
      real (kind=dbl_kind) ::
     &   rcKH             ! sqrt(cK*hin)                          (m**.5)
     &,  bet              ! 2*(cK-hin)                                (m)

      do nc = 1,ncat
         rcKH      = sqrt(cK*hin(nc))
         Hmean(nc) = hin(nc)+rcKH
         bet       = c2*(cK-hin(nc))

         if (rowflg(nc)) then
            n1 = nc  ! first guess
            n2 = n1  ! first guess
            do k = nc+1,ncat
               if (c2*hin(nc).ge.hin_max(k-1)) n1 = k
            enddo
            if (n1.eq.ncat) then
               ! case when all ice ridges into thickest cat
               Gamm(nc,n1)   = hin(nc)/Hmean(nc)
               HGamm(nc,n1)  = hin(nc)
               H2Gamm(nc,n1) = hin(nc)*Hmean(nc)
            elseif (2*rcKH.le.hin_max(n1)) then
               ! case when all ice ridges into just one cat
               Gamm(nc,n1)   = hin(nc)/Hmean(nc)
               HGamm(nc,n1)  = hin(nc)
               H2Gamm(nc,n1) = hin(nc)*Hmean(nc)
            else
               ! general case
               n2 = ncat
               do k = ncat-1,n1+1,-1
                  if (c2*rcKH.lt.hin_max(k)) n2 = k
               enddo
               Gamm(nc,n1)   = (hin_max(n1)-c2*hin(nc))/bet
               HGamm(nc,n1)  = Gamm(nc,n1)*(hin(nc)+hin_max(n1)/c2)
               H2Gamm(nc,n1) = HGamm(nc,n1)*(hin(nc)+hin_max(n1)/c2)

               Gamm(nc,n2)   = (c2*rcKH-hin_max(n2-1))/bet
               HGamm(nc,n2)  = Gamm(nc,n2)*(hin_max(n2-1)/c2+rcKH)
               H2Gamm(nc,n2) = HGamm(nc,n2)*(hin_max(n2-1)/c2+rcKH)

               do k = n1+1,n2-1
                 Gamm(nc,k)   = (hin_max(k)-hin_max(k-1))/bet
                 HGamm(nc,k)  = Gamm(nc,k)*hin_mp(k) 
                 H2Gamm(nc,k) = HGamm(nc,k)*hin_mp(k) 
               enddo
            endif
         endif
      enddo

      end subroutine comp_matrices
!
!BOP
!
! !IROUTINE: real function roth_strength - Rothrock ice strength
!
! !INTERFACE:
!
      real function roth_strength( ai0, ain, vin, Gamm ,H2Gamm )
!
! !DESCRIPTION:
!
! Compute the ice strength based on Rothrock, 1975 and also see FH95 \\
! does not make sense to do this unless the ice that participates in \\
! ridging is well resolved -- must have about 5 categories, 10 would
! be better
!
! !REVISION HISTORY:
!
! author: C.M.Bitz, UW
!
! !USES:
!
! !INPUT/OUTPUT PARAMETERS:
!
      real (kind=dbl_kind), intent(in) ::
     &   Gamm    (ncat,ncat) ! area frac from cat i that goes into j
     &,  H2Gamm  (ncat,ncat) ! H*volume frac from cat i that goes into j
     &,  ai0                 ! open water area JLS
     &,  ain      (ncat)      ! Ice fraction JLS
     &,  vin      (ncat)      ! Ice vol JLS
!
!EOP
!
      integer (kind=int_kind) :: k,nc
      real (kind=dbl_kind) ::
     &   W       (0:ncat)    ! W factor, geometric comp. of ridg mode 
     &,  Wa      (0:ncat)    ! part of W from participation func.
     &,  hin      (ncat)      ! ice thickness                           (m)
     &,  peterm              ! contribution from potential energy   (m**2)

      roth_strength = c0
      if (ai0.ge.gstar) return

      ! strictly speaking, should recompute Gamm and H2Gamm matrices here,
      ! but they would be only slightly different from what they are
      ! now and it does not affect conservation prop.

      call ridging_mode( ai0, ain, W ,Wa ,Gamm )

      do nc = 1,ncat
         if ( (ain(nc).gt.puny) .and. (vin(nc).gt.0.) )  then
            hin(nc) = vin(nc)/ain(nc)
         else
            hin(nc) = max(hin_max(nc-1),hi_min)
         endif
      enddo

      peterm = c0
      do nc = 1,ncat
         peterm = peterm - Wa(nc)*hin(nc)*hin(nc)
         do k = 1,nc
            peterm = peterm + Wa(k)*H2Gamm(k,nc)
         enddo
      enddo
      peterm = max(c0,peterm)

      ! factor of Zfric accounts for frictional energy
      roth_strength = peterm*cpe*Zfric  

      end function roth_strength
!
      end module ice_mechred
!
